Index: if_lagg.c
===================================================================
--- if_lagg.c	(revision 267666)
+++ if_lagg.c	(working copy)
@@ -189,6 +189,10 @@
 SYSCTL_INT(_net_link_lagg, OID_AUTO, default_flowid_shift, CTLFLAG_RW,
     &def_flowid_shift, 0,
     "Default setting for flowid shift for load sharing");
+static int lagg_rr_packets = 0; /* Default value for using rr_packets */
+SYSCTL_INT(_net_link_lagg, OID_AUTO, rr_packets, CTLFLAG_RW,
+    &lagg_rr_packets, 0,
+    "How many packets to be send per interface");
 
 static int
 lagg_modevent(module_t mod, int type, void *data)
@@ -1689,14 +1693,68 @@
 {
 	struct lagg_port *lp;
 	uint32_t p;
+	uint32_t pkt_sysctl_count;
+	int ifp_count = 1;
 
 	p = atomic_fetchadd_32(&sc->sc_seq, 1);
 	p %= sc->sc_count;
 	lp = SLIST_FIRST(&sc->sc_ports);
-	while (p--)
-		lp = SLIST_NEXT(lp, lp_entries);
 
 	/*
+	 * If there is no reference for the IFP, we must
+ 	 * copy it now.
+	 */
+	if (strlen(sc->sc_ref_ifp) == 0)
+		strncpy(sc->sc_ref_ifp, lp->lp_ifp->if_xname, sizeof(sc->sc_ref_ifp));
+              
+	/*
+	 * If ifp_count was not yet initialized, we must
+	 * initialize now.
+	 */
+	if (sc->sc_ifp_count == 0)
+		sc->sc_ifp_count = 1;
+
+	/*
+	 * If the sysctl rr_packets is set to 0, we must use the
+	 * roundrobin as it is, or otherwise, we must apply the
+	 * granularity between the interfaces that are part of the group.
+	 */
+	if (!lagg_rr_packets) {
+		while (p--)
+			lp = SLIST_NEXT(lp, lp_entries);
+		goto send_mbuf;
+	} else {
+		pkt_sysctl_count = atomic_fetchadd_32(&sc->sc_pkt_count, 1);
+		if (pkt_sysctl_count == lagg_rr_packets) {
+			if (sc->sc_ifp_count <= sc->sc_count) {
+				while (ifp_count < sc->sc_ifp_count) {
+					lp = SLIST_NEXT(lp, lp_entries);
+					ifp_count++;
+				}
+				sc->sc_ifp_count++;
+				if (sc->sc_ifp_count > sc->sc_count)
+					sc->sc_ifp_count = 0;
+			}
+			strncpy(sc->sc_ref_ifp, lp->lp_ifp->if_xname, sizeof(sc->sc_ref_ifp));
+			sc->sc_pkt_count = 0;
+		}
+	}
+
+	/*
+	 * Check if the current interface to be enqueue is not the
+	 * same used in the last round.
+	 */
+	lp = SLIST_FIRST(&sc->sc_ports);
+	for (;;) {
+		if (strcmp(lp->lp_ifp->if_xname, sc->sc_ref_ifp) == 0)
+			break;
+		else
+			lp = SLIST_NEXT(lp, lp_entries);
+	}
+	goto send_mbuf;
+
+send_mbuf:
+	/*
 	 * Check the port's link state. This will return the next active
 	 * port if the link is down or the port is NULL.
 	 */
Index: if_lagg.h
===================================================================
--- if_lagg.h	(revision 267666)
+++ if_lagg.h	(working copy)
@@ -232,6 +232,9 @@
 	struct sysctl_oid		*sc_oid;	/* sysctl tree oid */
 	int				use_flowid;	/* use M_FLOWID */
 	int				flowid_shift;	/* shift the flowid */
+	uint32_t			sc_pkt_count; /* use for count packates per ifp */
+	int				sc_ifp_count; /* counter reference of interfaces on rr */
+	char				sc_ref_ifp[IFNAMSIZ]; /* name of the ifp */
 };
 
 struct lagg_port {
